==> xt-cvdata-0.5.0.tar.gz: xt-cvdata-0.5.0/xt_cvdata/transforms/transforms.py <==

import torch 
import numpy as np
from PIL import Image
from torchvision.transforms import functional as F
import random

__all__ = ['Relabel', 'ToLabel', 'Squeeze', 'OneHot', 'MaskCompose', 
    'RandomHorizontalFlipSeg', 'RandomResizedCropSeg'
]

class MaskCompose:
    """ Custom Composer of transforms used for applying transforms to the img, mask, or both.

    MaskCompose([(Resize(24, 24), 'both'), (ColorJitter(), 'img'), (lambda x: x.long(), 'mask')])
    """
    target_options = ['img', 'mask', 'both']
    def __init__(self, transforms):
        self.transforms = transforms

        assert all([target in self.target_options for _, target in transforms])
    
    def __call__(self, img, mask):
        for t, target in self.transforms:
            if target == 'img':
                img = t(img)
            elif target == 'mask':
                mask = t(mask)
            else:
                # Concat then transform then split
                img, mask = t(img, mask)
                
        return img, mask


class Relabel:
    """ Transform to replace all occurances of `old_label` with `new_label` in a torch.tensor.
    
    Relabel(old_label, new_label)
    """

    def __init__(self, olabel, nlabel):
        self.olabel = olabel
        self.nlabel = nlabel

    def __call__(self, tensor):
        assert isinstance(tensor, torch.LongTensor), 'tensor needs to be LongTensor'
        tensor[tensor == self.olabel] = self.nlabel
        return tensor

class ToLabel:
    def __call__(self, image):
        return torch.from_numpy(np.array(image)).long().unsqueeze(0)

class Squeeze:
    def __init__(self, dim=None):
        self.dim = dim

    def __call__(self, x):
        return torch.squeeze(x, dim=self.dim)

class OneHot:

    def __init__(self, C):
        self.C = C

    def __call__(self, labels):
        '''
        Converts an integer label torch.autograd.Variable to a one-hot Variable.
        
        Parameters
        ----------
        labels : torch.autograd.Variable of torch.cuda.LongTensor
            N x 1 x H x W, where N is batch size. 
            Each value is an integer representing correct classification.
        C : integer. 
            number of classes in labels.
        
        Returns
        -------
        target : torch.autograd.Variable of torch.cuda.FloatTensor
            N x C x H x W, where C is class number. One-hot encoded.
        '''


        one_hot = torch.LongTensor(self.C, labels.size(1), labels.size(2)).zero_()
        target = one_hot.scatter_(0, labels.data, 1)
        
        target = torch.autograd.Variable(target)
            
        return target

class RandomResizedCropSeg:
    """Crop the given PIL Image to random size and aspect ratio.

    A crop of random size (default: of 0.08 to 1.0) of the original size and a random
    aspect ratio (default: of 3/4 to 4/3) of the original aspect ratio is made. This crop
    is finally resized to given size.
    This is popularly used to train the Inception networks.

    Args:
        size: expected output size of each edge
        scale: range of size of the origin size cropped
        ratio: range of aspect ratio of the origin aspect ratio cropped
        interpolation: Default: PIL.Image.BILINEAR
    """

    def __init__(self, size, scale=(0.08, 1.0), ratio=(3. / 4., 4. / 3.), interpolation=Image.BILINEAR):
        if isinstance(size, (tuple, list)):
            self.size = size
        else:
            self.size = (size, size)
        if (scale[0] > scale[1]) or (ratio[0] > ratio[1]):
            warnings.warn("range should be of kind (min, max)")

        self.interpolation = interpolation
        self.scale = scale
        self.ratio = ratio

    @staticmethod
    def get_params(img, scale, ratio):
        """Get parameters for ``crop`` for a random sized crop.

        Args:
            img (PIL Image): Image to be cropped.
            scale (tuple): range of size of the origin size cropped
            ratio (tuple): range of aspect ratio of the origin aspect ratio cropped

        Returns:
            tuple: params (i, j, h, w) to be passed to ``crop`` for a random
                sized crop.
        """
        width, height = _get_image_size(img)
        area = height * width

        for _ in range(10):
            target_area = random.uniform(*scale) * area
            log_ratio = (math.log(ratio[0]), math.log(ratio[1]))
            aspect_ratio = math.exp(random.uniform(*log_ratio))

            w = int(round(math.sqrt(target_area * aspect_ratio)))
            h = int(round(math.sqrt(target_area / aspect_ratio)))

            if 0 < w <= width and 0 < h <= height:
                i = random.randint(0, height - h)
                j = random.randint(0, width - w)
                return i, j, h, w

        # Fallback to central crop
        in_ratio = float(width) / float(height)
        if (in_ratio < min(ratio)):
            w = width
            h = int(round(w / min(ratio)))
        elif (in_ratio > max(ratio)):
            h = height
            w = int(round(h * max(ratio)))
        else:  # whole image
            w = width
            h = height
        i = (height - h) // 2
        j = (width - w) // 2
        return i, j, h, w

    def __call__(self, img, mask):
        """
        Args:
            img (PIL Image): Image to be cropped and resized.

        Returns:
            PIL Image: Randomly cropped and resized image.
        """
        i, j, h, w = self.get_params(img, self.scale, self.ratio)
        return (
            F.resized_crop(img, i, j, h, w, self.size, self.interpolation),
            F.resized_crop(mask, i, j, h, w, self.size, self.interpolation)
        )

class RandomHorizontalFlipSeg(object):
    """Horizontally flip the given PIL Image randomly with a given probability.

    Args:
        p (float): probability of the image being flipped. Default value is 0.5
    """

    def __init__(self, p=0.5):
        self.p = p

    def __call__(self, img, mask):
        """
        Args:
            img (PIL Image): Image to be flipped.

        Returns:
            PIL Image: Randomly flipped image.
        """
        if random.random() < self.p:
            return (
                F.hflip(img),
                F.hflip(mask)
            )
        return img, mask

    def __repr__(self):
        return self.__class__.__name__ + '(p={})'.format(self.p)

==> xt-cvdata-0.5.1.tar.gz: xt-cvdata-0.5.1/xt_cvdata/transforms/transforms.py <==

import torch 
import numpy as np
from PIL import Image
from torchvision.transforms import functional as F
import random

__all__ = ['Relabel', 'ToLabel', 'Squeeze', 'OneHot', 'MaskCompose', 
    'RandomHorizontalFlipSeg', 'RandomResizedCropSeg'
]

class MaskCompose:
    """ Custom Composer of transforms used for applying transforms to the img, mask, or both.

    MaskCompose([(Resize(24, 24), 'both'), (ColorJitter(), 'img'), (lambda x: x.long(), 'mask')])
    """
    target_options = ['img', 'mask', 'both']
    def __init__(self, transforms):
        self.transforms = transforms

        assert all([target in self.target_options for _, target in transforms])
    
    def __call__(self, img, mask):
        for t, target in self.transforms:
            if target == 'img':
                img = t(img)
            elif target == 'mask':
                mask = t(mask)
            else:
                # Concat Mask, Transform, Split
                img.putalpha(mask)
                img = t(img)
                red, green, blue, mask = img.split()
                img = Image.merge('RGB', [red, green, blue])

        return img, mask


class Relabel:
    """ Transform to replace all occurances of `old_label` with `new_label` in a torch.tensor.
    
    Relabel(old_label, new_label)
    """

    def __init__(self, olabel, nlabel):
        self.olabel = olabel
        self.nlabel = nlabel

    def __call__(self, tensor):
        assert isinstance(tensor, torch.LongTensor), 'tensor needs to be LongTensor'
        tensor[tensor == self.olabel] = self.nlabel
        return tensor

class ToLabel:
    def __call__(self, image):
        return torch.from_numpy(np.array(image)).long().unsqueeze(0)

class Squeeze:
    def __init__(self, dim=None):
        self.dim = dim

    def __call__(self, x):
        return torch.squeeze(x, dim=self.dim)

class OneHot:

    def __init__(self, C):
        self.C = C

    def __call__(self, labels):
        '''
        Converts an integer label torch.autograd.Variable to a one-hot Variable.
        
        Parameters
        ----------
        labels : torch.autograd.Variable of torch.cuda.LongTensor
            N x 1 x H x W, where N is batch size. 
            Each value is an integer representing correct classification.
        C : integer. 
            number of classes in labels.
        
        Returns
        -------
        target : torch.autograd.Variable of torch.cuda.FloatTensor
            N x C x H x W, where C is class number. One-hot encoded.
        '''


        one_hot = torch.LongTensor(self.C, labels.size(1), labels.size(2)).zero_()
        target = one_hot.scatter_(0, labels.data, 1)
        
        target = torch.autograd.Variable(target)
            
        return target

class RandomResizedCropSeg:
    """Crop the given PIL Image and mask to random size and aspect ratio.

    A crop of random size (default: of 0.08 to 1.0) of the original size and a random
    aspect ratio (default: of 3/4 to 4/3) of the original aspect ratio is made. This crop
    is finally resized to given size.
    This is popularly used to train the Inception networks.

    Args:
        size: expected output size of each edge
        scale: range of size of the origin size cropped
        ratio: range of aspect ratio of the origin aspect ratio cropped
        interpolation: Default: PIL.Image.BILINEAR
    """

    def __init__(self, size, scale=(0.08, 1.0), ratio=(3. / 4., 4. / 3.), interpolation=Image.BILINEAR):
        if isinstance(size, (tuple, list)):
            self.size = size
        else:
            self.size = (size, size)
        if (scale[0] > scale[1]) or (ratio[0] > ratio[1]):
            warnings.warn("range should be of kind (min, max)")

        self.interpolation = interpolation
        self.scale = scale
        self.ratio = ratio

    @staticmethod
    def get_params(img, scale, ratio):
        """Get parameters for ``crop`` for a random sized crop.

        Args:
            img (PIL Image): Image to be cropped.
            scale (tuple): range of size of the origin size cropped
            ratio (tuple): range of aspect ratio of the origin aspect ratio cropped

        Returns:
            tuple: params (i, j, h, w) to be passed to ``crop`` for a random
                sized crop.
        """
        width, height = _get_image_size(img)
        area = height * width

        for _ in range(10):
            target_area = random.uniform(*scale) * area
            log_ratio = (math.log(ratio[0]), math.log(ratio[1]))
            aspect_ratio = math.exp(random.uniform(*log_ratio))

            w = int(round(math.sqrt(target_area * aspect_ratio)))
            h = int(round(math.sqrt(target_area / aspect_ratio)))

            if 0 < w <= width and 0 < h <= height:
                i = random.randint(0, height - h)
                j = random.randint(0, width - w)
                return i, j, h, w

        # Fallback to central crop
        in_ratio = float(width) / float(height)
        if (in_ratio < min(ratio)):
            w = width
            h = int(round(w / min(ratio)))
        elif (in_ratio > max(ratio)):
            h = height
            w = int(round(h * max(ratio)))
        else:  # whole image
            w = width
            h = height
        i = (height - h) // 2
        j = (width - w) // 2
        return i, j, h, w

    def __call__(self, img, mask):
        """
        Args:
            img (PIL Image): Image to be cropped and resized.

        Returns:
            PIL Image: Randomly cropped and resized image.
        """
        i, j, h, w = self.get_params(img, self.scale, self.ratio)
        return (
            F.resized_crop(img, i, j, h, w, self.size, self.interpolation),
            F.resized_crop(mask, i, j, h, w, self.size, self.interpolation)
        )

class RandomHorizontalFlipSeg(object):
    """Horizontally flip the given PIL Image and mask randomly with a given probability.

    Args:
        p (float): probability of the image being flipped. Default value is 0.5
    """

    def __init__(self, p=0.5):
        self.p = p

    def __call__(self, img, mask):
        """
        Args:
            img (PIL Image): Image to be flipped.

        Returns:
            PIL Image: Randomly flipped image.
        """
        if random.random() < self.p:
            return (
                F.hflip(img),
                F.hflip(mask)
            )
        return img, mask

    def __repr__(self):
        return self.__class__.__name__ + '(p={})'.format(self.p)
